# -*- perl -*-
#
# Copyright (C) 1996 Ken MacLeod
# See the file COPYING for distribution terms.
#

use strict;

package Text::GDO::Context;

#
# `initted' is true after the first `string_context'
#
$Text::GDO::Context::initted_entities = 0;

sub new {
    my ($type, %value) = @_;

    my ($self) = \%value;

    if (!defined $self->{'current'}) {
	$self->{'current'} = [{'fill' => 0}];
    }

    bless $self, $type;
}

#
# Stolen from toAscii.
# XXX can this be share with toAscii? cascading?
# XXX can we cascade all the entities regardless of context?
# XXX does this need to be a mapping from ISO entities to a standard char set?
#
sub _init {
    my ($prefix, $sdatadir, $file_name);

    foreach $prefix (@::INC) {
	$sdatadir = "$prefix/sdata";
	if (-d $sdatadir) {
	    opendir (SDATADIR, $sdatadir)
		|| die "$::prog: Can't open directory \`$sdatadir' for reading: $!\n";
	    foreach $file_name (readdir (SDATADIR)) {
		next if ($file_name !~ /\.2ab$/);
		open (ENTITYFILE, "$sdatadir/$file_name")
		    || die "$::prog: Can't open \`$sdatadir/$file_name' for reading: $!\n";
		while (<ENTITYFILE>) {
		    chop;
		    s/(^[^\t]+)\t(.*)/$Text::GDO::Context::entities{"$1"} = $2/e;
		}
		close (ENTITYFILE);
	    }
	    closedir (SDATADIR);
	}
    }

    # XXX fix in sdata
    $Text::GDO::Context::entities{"[bull  ]"} = "*";
    $Text::GDO::Context::entities{"[tilde ]"} = "~";

    # XXX LinuxDoc, should be fixed in the linuxdoc-dtd patch
    $Text::GDO::Context::entities{"</"} = "</";

    $Text::GDO::Context::initted_entities = 1;
}

sub string_context {
    my (@args) = @_;

    # XXX can we know if this context can be made static?
    Text::GDO::Context::_init() if (!$Text::GDO::Context::initted_entities);
    my ($self) = new Text::GDO::Context ('entities' => \%Text::GDO::Context::entities, @args);

    return $self;
}
    

sub push {
    my ($self, %new_values) = @_;
    my ($key, $value, $new);

    $new = $self->copy;
    while (($key, $value) = each %new_values) {
	$new->{$key} = $value;
    }
    push (@{$self->{'current'}}, $new);
}

sub pop {
    my ($self) = @_;

    pop (@{$self->{'current'}});
}

sub copy {
    my ($self) = @_;
    my ($key, $value);
    my ($new_current) = {};

    while (($key, $value) = each %{$self->{'current'}[-1]}) {
	$new_current->{$key} = $value;
    }

    $new_current;
}

sub reference  {
    my ($self) = shift;

    return $self->{'rootnode'}->referred (@_);
}

sub charent {
    my ($context, $charent) = @_;

    if (defined $context->{'entities'}{$charent}) {
	return ($context->{'entities'}{$charent});
    } elsif (defined $context->{'rootnode'}{'entities'}{$charent}) {
	return $context->{'rootnote'}{'entities'}{$charent};
    } else {
	if (!defined $context->{'warned_entity'}{$charent}) {
	    $context->{'warned_entity'}{$charent} = 1;
	    warn "$prog:: entity \`$charent' not defined\n";
	}
	return $charent;
    }
}

sub fill      { return $_[0]->{'current'}[-1]{'fill'}; }

sub set_fill      { return $_[0]->{'current'}[-1]{'fill'} = $_[1]; }

1;
